@@ -1,15 +1,16 @@
 #!/usr/bin/env python3
 """Main CLI handler module for CommitLoom."""
 
-import logging
 import os
 import subprocess
 import sys
+from datetime import datetime
 
 from dotenv import load_dotenv
 
-from ..core.analyzer import CommitAnalyzer
+from ..core.analyzer import CommitAnalysis, CommitAnalyzer
 from ..core.git import GitError, GitFile, GitOperations
+from ..core.smart_grouping import SmartGrouper
 from ..services.ai_service import AIService
 from ..services.metrics import metrics_manager  # noqa
 from . import console
@@ -17,14 +18,8 @@ from . import console
 env_path = os.path.join(os.path.dirname(__file__), "..", "..", ".env")
 load_dotenv(dotenv_path=env_path)
 
-# Configure logging
-logging.basicConfig(
-    level=logging.INFO,
-    format="%(levelname)s: %(message)s",
-    handlers=[logging.StreamHandler(sys.stdout)],
-)
-
-logger = logging.getLogger(__name__)
+# Logging is configured by console module
+# logger = logging.getLogger(__name__)
 
 # Minimum number of files to activate batch processing
 BATCH_THRESHOLD = 3
@@ -45,20 +40,34 @@ class CommitLoom:
             self.repo_path = os.path.basename(os.getcwd())
         except Exception:
             self.repo_path = "unknown_repo"
-            
+
         self.git = GitOperations()
         self.analyzer = CommitAnalyzer()
         self.ai_service = AIService(api_key=api_key, test_mode=test_mode)
+        self.smart_grouper = SmartGrouper()
         self.auto_commit = False
         self.combine_commits = False
         self.console = console
+        self.use_smart_grouping = True  # Flag to enable/disable smart grouping
+
+    def _maybe_create_branch(self, analysis: CommitAnalysis) -> None:
+        """Offer to create a new branch if the commit is complex."""
+        if not analysis.is_complex:
+            return
+        branch_name = f"loom-large-{datetime.now().strftime('%Y%m%d_%H%M%S')}"
+        if console.confirm_branch_creation(branch_name):
+            try:
+                self.git.create_and_checkout_branch(branch_name)
+                console.print_info(f"Switched to new branch {branch_name}")
+            except GitError as e:
+                console.print_error(str(e))
 
     def _process_single_commit(self, files: list[GitFile]) -> None:
         """Process files as a single commit."""
         try:
             # Start tracking metrics
             metrics_manager.start_commit_tracking(repository=self.repo_path)
-            
+
             # Stage files
             file_paths = [f.path for f in files]
             self.git.stage_files(file_paths)
@@ -69,6 +78,7 @@ class CommitLoom:
 
             # Print analysis
             console.print_warnings(analysis)
+            self._maybe_create_branch(analysis)
 
             try:
                 # Generate commit message
@@ -92,15 +102,15 @@ class CommitLoom:
                     prompt_tokens=usage.prompt_tokens,
                     completion_tokens=usage.completion_tokens,
                     cost_in_eur=usage.total_cost,
-                    model_used=self.ai_service.model
+                    model_used=self.ai_service.model,
                 )
                 sys.exit(0)
 
             # Create commit
             commit_success = self.git.create_commit(suggestion.title, suggestion.format_body())
             if commit_success:
-                console.print_success("Changes committed successfully!")
-                
+                console.print_success("Changes committed successfully!", show_quote=True)
+
                 # Record metrics
                 metrics_manager.finish_commit_tracking(
                     files_changed=len(files),
@@ -108,12 +118,12 @@ class CommitLoom:
                     prompt_tokens=usage.prompt_tokens,
                     completion_tokens=usage.completion_tokens,
                     cost_in_eur=usage.total_cost,
-                    model_used=self.ai_service.model
+                    model_used=self.ai_service.model,
                 )
             else:
                 console.print_warning("No changes were committed. Files may already be committed.")
                 self.git.reset_staged_changes()
-                
+
                 # Record metrics with 0 files
                 metrics_manager.finish_commit_tracking(
                     files_changed=0,
@@ -121,7 +131,7 @@ class CommitLoom:
                     prompt_tokens=usage.prompt_tokens,
                     completion_tokens=usage.completion_tokens,
                     cost_in_eur=usage.total_cost,
-                    model_used=self.ai_service.model
+                    model_used=self.ai_service.model,
                 )
                 sys.exit(0)
 
@@ -144,7 +154,7 @@ class CommitLoom:
         try:
             # Start tracking metrics
             metrics_manager.start_commit_tracking(repository=self.repo_path)
-            
+
             # Stage files
             file_paths = [f.path for f in batch]
             self.git.stage_files(file_paths)
@@ -173,7 +183,7 @@ class CommitLoom:
             if not commit_success:
                 console.print_warning("No changes were committed. Files may already be committed.")
                 self.git.reset_staged_changes()
-                
+
                 # Record metrics with 0 files
                 metrics_manager.finish_commit_tracking(
                     files_changed=0,
@@ -184,7 +194,7 @@ class CommitLoom:
                     model_used=self.ai_service.model,
                     batch_processing=True,
                     batch_number=batch_num,
-                    batch_total=total_batches
+                    batch_total=total_batches,
                 )
                 return None
 
@@ -198,7 +208,7 @@ class CommitLoom:
                 model_used=self.ai_service.model,
                 batch_processing=True,
                 batch_number=batch_num,
-                batch_total=total_batches
+                batch_total=total_batches,
             )
 
             console.print_batch_complete(batch_num, total_batches)
@@ -224,9 +234,7 @@ class CommitLoom:
             invalid_files = []
 
             for file in changed_files:
-                if hasattr(self.git, "should_ignore_file") and self.git.should_ignore_file(
-                    file.path
-                ):
+                if self.git.should_ignore_file(file.path):
                     invalid_files.append(file)
                     console.print_warning(f"Ignoring file: {file.path}")
                 else:
@@ -236,19 +244,50 @@ class CommitLoom:
                 console.print_warning("No valid files to process.")
                 return []
 
-            # Create batches from valid files
-            batches = []
-            batch_size = BATCH_THRESHOLD
-            for i in range(0, len(valid_files), batch_size):
-                batch = valid_files[i : i + batch_size]
-                batches.append(batch)
-
-            return batches
+            # Use smart grouping if enabled
+            if self.use_smart_grouping:
+                return self._create_smart_batches(valid_files)
+            else:
+                # Fallback to basic grouping
+                return self._create_basic_batches(valid_files)
 
         except subprocess.CalledProcessError as e:
             console.print_error(f"Error getting git status: {e}")
             return []
 
+    def _create_basic_batches(self, valid_files: list[GitFile]) -> list[list[GitFile]]:
+        """Create basic batches using the old grouping logic."""
+        # Group files by top-level directory for smarter batching
+        grouped: dict[str, list[GitFile]] = {}
+        for f in valid_files:
+            parts = f.path.split(os.sep)
+            top_dir = parts[0] if len(parts) > 1 else "root"
+            grouped.setdefault(top_dir, []).append(f)
+
+        batches = []
+        batch_size = BATCH_THRESHOLD
+        for group_files in grouped.values():
+            for i in range(0, len(group_files), batch_size):
+                batches.append(group_files[i : i + batch_size])
+
+        return batches
+
+    def _create_smart_batches(self, valid_files: list[GitFile]) -> list[list[GitFile]]:
+        """Create intelligent batches using semantic analysis."""
+        # Use the smart grouper to analyze files
+        file_groups = self.smart_grouper.analyze_files(valid_files)
+
+        if not file_groups:
+            console.print_warning("Smart grouping produced no groups, falling back to basic grouping")
+            return self._create_basic_batches(valid_files)
+
+        # Print concise group summary
+        console.print_info(f"Smart grouping created {len(file_groups)} groups based on file relationships")
+
+        # Convert FileGroup objects to lists of GitFile
+        batches = [group.files for group in file_groups]
+        return batches
+
     def _create_combined_commit(self, batches: list[dict]) -> None:
         """Create a combined commit from multiple batches."""
         try:
@@ -268,18 +307,12 @@ class CommitLoom:
 
             # Create combined commit message
             title = "📦 chore: combine multiple changes"
-            body = "\n\n".join(
-                [
-                    title,
-                    "\n".join(
-                        f"{data['emoji']} {category}:" for category, data in all_changes.items()
-                    ),
-                    "\n".join(
-                        f"- {change}" for data in all_changes.values() for change in data["changes"]
-                    ),
-                    " ".join(summary_points),
-                ]
-            )
+            body_parts = [
+                "\n".join(f"{data['emoji']} {category}:" for category, data in all_changes.items()),
+                "\n".join(f"- {change}" for data in all_changes.values() for change in data["changes"]),
+                " ".join(summary_points),
+            ]
+            body = "\n\n".join(part for part in body_parts if part)
 
             # Stage and commit all files
             self.git.stage_files(all_files)
@@ -288,7 +321,7 @@ class CommitLoom:
                 self.git.reset_staged_changes()
                 sys.exit(0)
 
-            console.print_success("Combined commit created successfully!")
+            console.print_success("Combined commit created successfully!", show_quote=True)
 
         except GitError as e:
             console.print_error(f"Git error: {str(e)}")
@@ -344,63 +377,73 @@ class CommitLoom:
         """Display usage statistics."""
         # Get usage statistics
         stats = metrics_manager.get_statistics()
-        
+
         console.console.print("\n[bold blue]📊 CommitLoom Usage Statistics[/bold blue]")
-        
+
         # Display basic stats
         console.console.print("\n[bold cyan]Basic Statistics:[/bold cyan]")
-        console.console.print(f"  • Total commits generated: {stats['total_commits']:,}")
-        console.console.print(f"  • Total tokens used: {stats['total_tokens']:,}")
-        console.console.print(f"  • Total cost: €{stats['total_cost_in_eur']:.4f}")
-        console.console.print(f"  • Total files processed: {stats['total_files_processed']:,}")
-        
+        console.console.print(f"  • Total commits generated: {stats.get('total_commits', 0):,}")
+        console.console.print(f"  • Total tokens used: {stats.get('total_tokens', 0):,}")
+        cost = stats.get("total_cost_in_eur", 0.0)
+        console.console.print(f"  • Total cost: €{cost:.4f}")
+        files = stats.get("total_files_processed", 0)
+        console.console.print(f"  • Total files processed: {files:,}")
+
         # Display time saved if available
-        if 'time_saved_formatted' in stats:
+        if "time_saved_formatted" in stats:
             console.console.print(f"  • Total time saved: {stats['time_saved_formatted']}")
-        
+
         # Display activity period if available
-        if 'first_used_at' in stats and stats['first_used_at'] and 'days_active' in stats:
-            console.console.print(f"  • Active since: {stats['first_used_at'].split('T')[0]}")
+        if "first_used_at" in stats and stats["first_used_at"] and "days_active" in stats:
+            first_used = stats["first_used_at"]
+            has_t = isinstance(first_used, str) and "T" in first_used
+            date_part = first_used.split("T")[0] if has_t else first_used
+            console.console.print(f"  • Active since: {date_part}")
             console.console.print(f"  • Days active: {stats['days_active']}")
-            
-            if 'avg_commits_per_day' in stats:
-                console.console.print(f"  • Average commits per day: {stats['avg_commits_per_day']:.2f}")
-                console.console.print(f"  • Average cost per day: €{stats['avg_cost_per_day']:.4f}")
-        
+
+            if "avg_commits_per_day" in stats:
+                avg_commits = stats["avg_commits_per_day"]
+                console.console.print(f"  • Average commits per day: {avg_commits:.2f}")
+                avg_cost = stats.get("avg_cost_per_day", 0.0)
+                console.console.print(f"  • Average cost per day: €{avg_cost:.4f}")
+
         # Display repository stats if available
-        if stats['repositories']:
+        repositories = stats.get("repositories", {})
+        if repositories and isinstance(repositories, dict):
             console.console.print("\n[bold cyan]Repository Activity:[/bold cyan]")
-            console.console.print(f"  • Most active repository: {stats['most_active_repository']}")
-            console.console.print(f"  • Repositories used: {len(stats['repositories'])}")
-        
+            if "most_active_repository" in stats and stats["most_active_repository"]:
+                most_active = stats["most_active_repository"]
+                console.console.print(f"  • Most active repository: {most_active}")
+            console.console.print(f"  • Repositories used: {len(repositories)}")
+
         # Display model usage if available
-        if stats['model_usage']:
+        model_usage = stats.get("model_usage", {})
+        if model_usage and isinstance(model_usage, dict):
             console.console.print("\n[bold cyan]Model Usage:[/bold cyan]")
-            for model, count in stats['model_usage'].items():
+            for model, count in model_usage.items():
                 console.console.print(f"  • {model}: {count} commits")
-        
+
         # Display batch vs single commits
         console.console.print("\n[bold cyan]Processing Methods:[/bold cyan]")
-        console.console.print(f"  • Batch commits: {stats['batch_commits']}")
-        console.console.print(f"  • Single commits: {stats['single_commits']}")
-        
+        console.console.print(f"  • Batch commits: {stats.get('batch_commits', 0)}")
+        console.console.print(f"  • Single commits: {stats.get('single_commits', 0)}")
+
         # Get more detailed stats if commits exist
-        if stats['total_commits'] > 0:
+        if stats.get("total_commits", 0) > 0:
             model_stats = metrics_manager.get_model_usage_stats()
             if model_stats:
                 console.console.print("\n[bold cyan]Detailed Model Stats:[/bold cyan]")
                 for model, model_data in model_stats.items():
                     console.console.print(f"  • {model}:")
-                    console.console.print(f"    - Total tokens: {model_data['tokens']:,}")
-                    console.console.print(f"    - Total cost: €{model_data['cost']:.4f}")
-                    console.console.print(f"    - Avg tokens per commit: {model_data['avg_tokens_per_commit']:.1f}")
-    
-    def run(
-        self, auto_commit: bool = False, combine_commits: bool = False, debug: bool = False
-    ) -> None:
+                    console.console.print(f"    - Total tokens: {model_data.get('tokens', 0):,}")
+                    cost = model_data.get("cost", 0.0)
+                    console.console.print(f"    - Total cost: €{cost:.4f}")
+                    avg_tokens = model_data.get("avg_tokens_per_commit", 0.0)
+                    console.console.print(f"    - Avg tokens per commit: {avg_tokens:.1f}")
+
+    def run(self, auto_commit: bool = False, combine_commits: bool = False, debug: bool = False) -> None:
         """Run the commit process."""
-        if debug:
-            self.console.setup_logging(debug)
+        # Logging is already configured in the main CLI callback
 
         # Set auto-confirm mode based on auto_commit flag
         console.set_auto_confirm(auto_commit)
